TEST_STATUS_NOT_EXECUTED = 0
TEST_STATUS_PASSED = 1
TEST_STATUS_FAULT = 2


class TestDesc:

    def __init__(self, test_class, params, name, full_name):
        self.test_class = test_class
        self.params = params
        self.name = name
        self.full_name = full_name
        self.suitable = False
        self.status = TEST_STATUS_NOT_EXECUTED
        self.log_text = None



class UserInteraction:

    def request_message(self, test, text):
        raise Exception("Method UserInteraction.request_message is not implemented")

    def request_yes_no(self, test, text):
        raise Exception("Method UserInteraction.request_yes_no is not implemented")

    def request_option(self, test, text, options):
        raise Exception("Method UserInteraction.request_option is not implemented")



import threading
import time
import os
import sys


class RumbootTestBase:
    timeout = 5 * 60
    requested = {} # { "param1": "value1", "param2": "value2", { ... } } is compared with the environment

    def _suitable(req, env):
        for key, value in req.items():
            if not key in env:
                return False
            if isinstance(value, dict):
                if not isinstance(env[key], dict):
                    return False
                return RumbootTestBase._suitable(value, env[key])
            if value != env[key]:
                return False
        return True

    @classmethod
    def suitable(self, env, params):
        return RumbootTestBase._suitable(self.requested, env)

    def __init__(self, name, full_name, terminal, reset_seq, env, params, user_interaction):
        self.name = name
        self.full_name = full_name
        self.terminal = terminal
        self.reset_seq = reset_seq
        self.env = env
        self.params = params
        self.user_interaction = user_interaction

    def run(self):
        try:
            self.reset_seq.resetToHost()
        except:
            print("WARN: Reset method doesn't support HOST mode switching")
            print("WARN: If things don't work - check jumpers!")
            self.reset_seq.reset()
        return True

    # UserInteraction
    def request_message(self, text):
        save_stdout = sys.stdout
        sys.stdout = sys.__stdout__
        threading.current_thread().wait_user = True
        try:
            self.user_interaction.request_message(self, text)
        finally:
            sys.stdout = save_stdout
            threading.current_thread.wait_user = False
        print(f"User interaction: {text}")

    # UserInteraction
    def request_yes_no(self, text):
        save_stdout = sys.stdout
        sys.stdout = sys.__stdout__
        threading.current_thread.wait_user = True
        try:
            result = self.user_interaction.request_yes_no(self, text)
        finally:
            sys.stdout = save_stdout
            threading.current_thread.wait_user = False
        answer = "YES" if result else "NO"
        print(f"User interaction: {text} - {answer}")
        return result

    # UserInteraction
    def request_option(self, text, options):
        save_stdout = sys.stdout
        sys.stdout = sys.__stdout__
        threading.current_thread.wait_user = True
        try:
            result = self.user_interaction.request_option(self, text, options)
        finally:
            sys.stdout = save_stdout
            threading.current_thread.wait_user = False
        answer = options[result]
        print(f"User interaction: {text} - {answer}")
        return result

    # ??? temporary -> terminal
    def write_command(self, cmd):
        cmd = cmd.encode() + b"\r"
        self.terminal.ser.write(cmd)


class UBootTestBase(RumbootTestBase):

    @classmethod
    def suitable(self, env, params):
        if not super(UBootTestBase, self).suitable(env, params):
            return False
        if not "uboot" in env:
            return False
        if not env["uboot"].get("active", False):
            return False
        return True

    def uboot_cmd_answer(self, cmd, last_line_answer):
        answer = self.terminal.shell_cmd(cmd)
        lines = answer.splitlines(keepends=False)
        last_line = "" if len(lines) == 0 else lines[-1]
        if last_line != last_line_answer:
            print(f"Expected answer: {last_line_answer}")
            print(f"Current answer: {last_line}")
            raise Exception("Unexpected answer")
        return True

    def mem_setup(self):
        mem_setup_cmd = self.env["uboot"].get("mem_setup_cmd")
        if mem_setup_cmd != None:
            self.terminal.shell_cmd(mem_setup_cmd)

    def mem_fill(self, addr, size, value):
        self.terminal.shell_cmd(f"mw.b {addr:X} {value:02X} {size:X}")

    def mem_random(self, addr, size):
        self.uboot_cmd_answer(f"rand {addr:X} {size:X}", f"{size} bytes filled with random data")

    def mem_cmp(self, addr1, addr2, size):
        self.uboot_cmd_answer(f"cmp.b {addr1:X} {addr2:X} {size:X}", f"Total of {size} byte(s) were the same")

    def uboot_upload_file(self, addr_as_text, file_path):
        transport = self.env["connection"]["transport"]
        if (transport == "edcl"):
            self.write_command(f"echo UPLOAD to {addr_as_text}. \\\'X\\\' for X-modem, \\\'E\\\' for EDCL")
            self.terminal.add_binaries(file_path)
            self.terminal.loop(False, True)
            self.terminal.wait_prompt()
            self.terminal.ser.write(b"\b") # ??? -> terminal clear E character
        elif (transport == "xmodem"):
            self.write_command(f"loadx {addr_as_text}")
            self.terminal.load_binaries(file_path)
        else:
            raise "Unsupported transport"

    def run(self):
        super().run()

        binaries = []
        if "spl_path" in self.env["uboot"]:
            binaries.append(os.path.join(self.env["root_path"], self.env["uboot"]["path_base"], self.env["uboot"]["spl_path"]))
        if "uboot_path" in self.env["uboot"]:
            binaries.append(os.path.join(self.env["root_path"], self.env["uboot"]["path_base"], self.env["uboot"]["uboot_path"]))
        for file in binaries:
            self.terminal.add_binaries(file)

        self.terminal.loop(False, True)
        self.terminal.shell_mode("=> ")
        self.terminal.wait_prompt()

        return True


class KernelTestBase(UBootTestBase):

    @classmethod
    def suitable(self, env, params):
        if not super(KernelTestBase, self).suitable(env, params):
            return False
        if not "kernel" in env:
            return False
        if not env["kernel"].get("active", False):
            return False
        return True

    def run(self):
        super().run()
        self.mem_setup()

        if "bootargs" in self.env["kernel"]:
            self.terminal.shell_cmd(f"setenv bootargs {self.env['kernel']['bootargs']}")

        uimage_path = os.path.join(self.env["root_path"], self.env["kernel"]["path_base"], self.env["kernel"]["uimage_path"])
        self.uboot_upload_file("${loadaddr}", uimage_path)

        dtb_path = os.path.join(self.env["root_path"], self.env["kernel"]["path_base"], self.env["kernel"]["dtb_path"])
        self.uboot_upload_file("${fdt_addr_r}", dtb_path)

        self.write_command("bootm ${loadaddr} - ${fdt_addr_r}")
        self.terminal.wait("{} login:")
        self.write_command(self.env["kernel"]["user"])
        self.terminal.wait("Password:")
        self.write_command(self.env["kernel"]["password"])
        self.terminal.wait("{}#")

        self.write_command('A="=="; PS1="$A> "')
        self.terminal.shell_mode("==> ")
        self.terminal.wait_prompt()

        return True



from rumboot.ops.base import base
from rumboot.ops.xfer import basic_uploader
import tqdm
import rumboot.xfer


class bootdownload(object):
    '''
    Hisilicon boot downloader

    >>> downloader = bootdownload()
    >>> downloader.burn(filename)

    '''
    def init_bootmode(self):
        i = 0
        counter = 0
        while i < 30:
            in_bin = self.s.read(1)
            if in_bin == b'\x00':
                continue
            if in_bin == b'\x20':
                counter = counter + 1
            if counter == 5:
                self.s.flushOutput()
                self.s.write(settings.AA)
                self.s.flushInput()
                print("Welcome to boot-mode\n")
                return None
            i = i + 1
        sys.exit(1)

    def __init__(self, chiptype, serialport='/dev/ttyUSB0'):
        print("Trying open", serialport)
        try:
            self.s = serial.Serial(
                port=serialport,
                baudrate=115200,
                parity=serial.PARITY_NONE,
                stopbits=serial.STOPBITS_ONE,
                bytesize=serial.EIGHTBITS,
                timeout=None
                #timeout = 1
            )
            self.init_bootmode()

        except serial.serialutil.SerialException:
            #no serial connection
            self.s = None
            print("\nFailed to open serial!", serialport)
            sys.exit(2)

        with open(chiptype+".json", "r") as read_file:
            data = json.load(read_file)
        print(data)
        self.chip = data

    def __del__(self):
        print("Exiting...")
        if self.s is not None:
            self.s.close()

    def calc_crc(self, data, crc=0):
        for char in data:
            crc = ((crc << 8) | char) ^ settings.crctable[(crc >> 8) & 0xff]
        for i in range(0, 2):
            crc = ((crc << 8) | 0) ^ settings.crctable[(crc >> 8) & 0xff]
        return crc & 0xffff

    def sendframe(self, data, loop):
        for i in range(1, loop):
            self.s.flushOutput()
            if self.DEBUG == 1:
                if len(data) > 20:
                    print(
                        "len: ",
                        len(data),
                        "write : [",
                        ''.join('%02x ' % i for i in data[:20]), "... ]"
                    )
                else:
                    print(
                        "len: ",
                        len(data),
                        "write : [",
                        ''.join('%02x ' % i for i in data), "]"
                    )
            self.s.write(data)
            self.s.flushInput()
            try:
                ack = self.s.read()
                if len(ack) == 1:
                    if self.DEBUG == 1:
                        print(
                            "ret ack   : ",
                            ''.join('0x%02x ' % i for i in ack)
                        )
                    if ack == settings.ack_b:
                        return None
            except:
                return None

        print('failed')

    def send_head_frame(self, length, address):
        print('Send HEAD frame...')
        self.s.timeout = 0.03

        frame = bytearray(14)
        frame[0] = settings.twos_complement_to_int(-2, 8)
        frame[1] = settings.twos_complement_to_int(0, 8)
        frame[2] = settings.twos_complement_to_int(-1, 8)
        frame[3] = settings.twos_complement_to_int(1, 8)

        frame[4] = (length >> 24) & 0xff
        frame[5] = (length >> 16) & 0xff
        frame[6] = (length >> 8) & 0xff
        frame[7] = (length) & 0xff
        frame[8] = (address >> 24) & 0xff
        frame[9] = (address >> 16) & 0xff
        frame[10] = (address >> 8) & 0xff
        frame[11] = (address) & 0xff
        crc = self.calc_crc(frame[:12])
        frame[12] = (crc >> 8) & 0xff
        frame[13] = crc & 0xff

        self.sendframe(frame, 16)

    def send_ddrstep(self):
        print('Send DDRSTEP frame...')
        seq = 1
        self.s.timeout = 0.15

        address0 = int(self.chip["ADDRESS"][0], 16)
        self.send_head_frame(64, address0)
        head = bytearray(3)
        head[0] = 0xDA  # b'\xDA'  # 0xDA
        head[1] = seq & 0xFF
        head[2] = (~seq) & 0xFF
        data = head + bytes(self.chip["DDRSTEP0"])
        crc = self.calc_crc(data)
        h = bytearray(2)
        h[0] = (crc >> 8) & 0xff
        h[1] = crc & 0xff
        data += h

        self.sendframe(data, 16)
        self.send_tail_frame(seq + 1)

    def send_tail_frame(self, seq):
        print('Send TAIL frame...')
        data = bytearray(3)
        data[0] = 0xED
        data[1] = seq & 0xFF
        data[2] = (~seq) & 0xFF

        crc = self.calc_crc(data)
        h = bytearray(2)
        h[0] = (crc >> 8) & 0xff
        h[1] = crc & 0xff
        data += h

        self.sendframe(data, 16)

    def send_data_frame(self, seq, data):
        self.s.timeout = 0.15
        head = bytearray(3)
        head[0] = 0xDA
        head[1] = seq & 0xFF
        head[2] = (~seq) & 0xFF
        data = head + data

        crc = self.calc_crc(data)
        h = bytearray(2)
        h[0] = (crc >> 8) & 0xff
        h[1] = crc & 0xff
        data += h

        self.sendframe(data, 32)

    def store_SPL(self, data):
        first_length = int(self.chip["FILELEN"][1], 16)
        address1 = int(self.chip["ADDRESS"][1], 16)
        self.send_head_frame(first_length, address1)

        bar = progressbar.ProgressBar(maxval=first_length, widgets=[
            'Send DATA frame',
            progressbar.Bar(left='[', marker='=', right=']'),
            progressbar.SimpleProgress(),
        ]).start() if self.DEBUG == 0 else None

        seq = 1
        data = data[:first_length]
        for chunk in (
            data[_:_+settings.MAX_DATA_LEN] for _ in range(
                0, first_length, settings.MAX_DATA_LEN
            )
        ):
            self.send_data_frame(
                seq,
                chunk
            )
            bar.update(seq*len(chunk)) if self.DEBUG == 0 else None
            seq += 1
        bar.finish() if self.DEBUG == 0 else None
        self.send_tail_frame(seq)

    def store_Uboot(self, data):
        length = len(data)
        address2 = int(self.chip["ADDRESS"][2], 16)
        self.send_head_frame(length, address2)

        bar = progressbar.ProgressBar(maxval=length, widgets=[
            'Send DATA frame',
            progressbar.Bar(left='[', marker='=', right=']'),
            progressbar.SimpleProgress(),
        ]).start() if self.DEBUG == 0 else None
        seq = 1
        for chunk in (
            data[_:_+settings.MAX_DATA_LEN] for _ in range(
                0, length, settings.MAX_DATA_LEN
            )
        ):
            self.send_data_frame(
                seq,
                chunk
            )
            bar.update(seq*len(chunk)) if self.DEBUG == 0 else None
            seq += 1
        bar.finish() if self.DEBUG == 0 else None
        self.send_tail_frame(seq)

    def send_data(self, data):
        self.send_ddrstep()
        self.store_SPL(data)
        self.store_Uboot(data)

    def burn(self, filename):
        f = open(filename, "rb")
        data = f.read()
        f.close()

        print('Sending', filename, '...')
        self.send_data(data)
        print('Done\n')

class hisi_chips_uploader(basic_uploader):
    def on_start(self):
        #HACK: Old chips don't print anything via uart
        #HACK: edcl is the only way to bring 'em up
        #HACK: We do the lucky guess by looking at the image header
        def prg(total_bytes, position, increment):
                self.term.progress.update(increment)     

        tp = self.term.formats.guess(self.term.runlist[0])
        if self.term.hack("silentRom"):
            print("WARNING: Bootloader doesn't support xmodem, forcing initial edcl upload")
            old = self.term.xfer.how
            self.term.xfer.selectTransport("edcl")
            if not self.term.xfer.push(self.term.chip.spl_address):
                return 1
            self.term.xfer.selectTransport(old)

        if not tp:
            return True

        if tp.name == "Legacy K1879XB1YA":
            self.term.xfer.write32(0x00100010, 0x00100014)
            return True
        if tp.name == "NM6408 (Legacy)":
            self.term.xfer.write32(0x30000, 0x1)
            return True


import os
import inspect
import fnmatch
import importlib
from rumboot.testing.test_desc import *


class TestRegistry:

    def __init__(self):
        self.all_tests = {} # { "test_name": TestDesc, "subdir": { ... } }
        self.test_root_path = os.path.abspath(os.path.curdir)

    def register_tests(self, test_module_path, test_class, params, name):
        if name == None:
            name = test_class.__name__
        if isinstance(params, dict):
            self._register_one_test(test_module_path, test_class, params, name)
        elif isinstance(params, list):
            index = 1
            for p in params:
                self._register_one_test(test_module_path, test_class, p, f"{name}:{index}")
                index += 1
        else:
            raise Exception("Test params must be dict or list")

    def test_iteration(self, func):
        self._test_iteration_recursive(self.all_tests, func)

    def _register_one_test(self, test_module_path, test_class, params, name):
        if test_class == None:
            raise Exception("Test class is not defined")
        relPath = os.path.relpath(os.path.normpath(test_module_path), self.test_root_path)
        pathStr = os.path.split(relPath)[0]
        path = pathStr.split(os.sep)
        if path == [""]:
            path = []
        self._add_test_recursive(path, self.all_tests, test_class, params, name, "")

    def _add_test_recursive(self, path, storage_dict, test_class, params, name, full_name):
        if path == []:
            if name in storage_dict:
                raise Exception(f"Test {name} already exists")
            full_name = full_name + ("." if full_name else "") + name
            storage_dict[name] = TestDesc(test_class, params, name, full_name)
        else:
            if path[0] not in storage_dict:
                storage_dict[path[0]] = {}
            if not isinstance(storage_dict[path[0]], dict):
                raise Exception(f"Test {path[0]} already exists")
            full_name = full_name + ("." if full_name else "") + path[0]
            self._add_test_recursive(path[1:], storage_dict[path[0]], test_class, params, name, full_name)

    def _test_iteration_recursive(self, tests, func):
        for key, value in tests.items():
            if isinstance(value, TestDesc):
                func(value)
            else:
                self._test_iteration_recursive(value, func)


test_registry = TestRegistry()


def rtest(params = {}, name = None):
    test_module_path = os.path.abspath(inspect.stack()[1][1])
    def decorator(test_class):
        test_registry.register_tests(test_module_path, test_class, params, name)
    return decorator


def register_test(test_class, params = {}, name = None):
    test_module_path = os.path.abspath(inspect.stack()[1][1])
    test_registry.register_tests(test_module_path, test_class, params, name)


def rumboot_test_directory(subdirName, filter = "test_*.py"):
    test_module_path = os.path.abspath(inspect.stack()[1][1])
    dir_module_path = os.path.split(test_module_path)[0]
    dir_path = os.path.join(dir_module_path, subdirName)
    for entry in os.scandir(dir_path):
        if entry.is_file and fnmatch.fnmatch(entry.name, filter):
            fullPath = os.path.join(dir_path, os.path.splitext(entry.name)[0])
            relPath = os.path.relpath(fullPath, test_registry.test_root_path)
            moduleName = relPath.replace(os.path.sep, ".")
            importlib.import_module(moduleName)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'testing_gui_main_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(912, 609)
        self.central_widget = QtWidgets.QWidget(MainWindow)
        self.central_widget.setObjectName("central_widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.central_widget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.central_widget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.verticalLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.test_selected_button = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.test_selected_button.setObjectName("test_selected_button")
        self.horizontalLayout_2.addWidget(self.test_selected_button)
        self.test_failed_button = QtWidgets.QPushButton(self.verticalLayoutWidget)
        self.test_failed_button.setObjectName("test_failed_button")
        self.horizontalLayout_2.addWidget(self.test_failed_button)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.test_tree = QtWidgets.QTreeWidget(self.verticalLayoutWidget)
        self.test_tree.setColumnCount(0)
        self.test_tree.setObjectName("test_tree")
        self.test_tree.header().setDefaultSectionSize(150)
        self.test_tree.header().setStretchLastSection(False)
        self.verticalLayout_2.addWidget(self.test_tree)
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(self.splitter)
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setObjectName("formLayout")
        self.label_3 = QtWidgets.QLabel(self.verticalLayoutWidget_2)
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.board_number_spin_box = QtWidgets.QSpinBox(self.verticalLayoutWidget_2)
        self.board_number_spin_box.setMaximumSize(QtCore.QSize(100, 16777215))
        self.board_number_spin_box.setAlignment(QtCore.Qt.AlignCenter)
        self.board_number_spin_box.setMaximum(1000000)
        self.board_number_spin_box.setObjectName("board_number_spin_box")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.board_number_spin_box)
        self.label_2 = QtWidgets.QLabel(self.verticalLayoutWidget_2)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.chip_name_line_edit = QtWidgets.QLineEdit(self.verticalLayoutWidget_2)
        self.chip_name_line_edit.setMouseTracking(False)
        self.chip_name_line_edit.setFocusPolicy(QtCore.Qt.NoFocus)
        self.chip_name_line_edit.setAlignment(QtCore.Qt.AlignCenter)
        self.chip_name_line_edit.setReadOnly(True)
        self.chip_name_line_edit.setObjectName("chip_name_line_edit")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.chip_name_line_edit)
        self.verticalLayout_5.addLayout(self.formLayout)
        self.current_test_tab_widget = QtWidgets.QTabWidget(self.verticalLayoutWidget_2)
        self.current_test_tab_widget.setObjectName("current_test_tab_widget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.current_test_description_label = QtWidgets.QLabel(self.tab)
        self.current_test_description_label.setAlignment(QtCore.Qt.AlignCenter)
        self.current_test_description_label.setWordWrap(True)
        self.current_test_description_label.setObjectName("current_test_description_label")
        self.verticalLayout_4.addWidget(self.current_test_description_label)
        self.current_test_tab_widget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.current_test_log_plain_text_edit = QtWidgets.QPlainTextEdit(self.tab_2)
        self.current_test_log_plain_text_edit.setUndoRedoEnabled(False)
        self.current_test_log_plain_text_edit.setReadOnly(True)
        self.current_test_log_plain_text_edit.setObjectName("current_test_log_plain_text_edit")
        self.verticalLayout_3.addWidget(self.current_test_log_plain_text_edit)
        self.current_test_tab_widget.addTab(self.tab_2, "")
        self.verticalLayout_5.addWidget(self.current_test_tab_widget)
        self.verticalLayout.addWidget(self.splitter)
        MainWindow.setCentralWidget(self.central_widget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 912, 20))
        self.menubar.setObjectName("menubar")
        self.menu = QtWidgets.QMenu(self.menubar)
        self.menu.setObjectName("menu")
        self.settings = QtWidgets.QMenu(self.menubar)
        self.settings.setObjectName("settings")
        self.menu_info = QtWidgets.QMenu(self.menubar)
        self.menu_info.setObjectName("menu_info")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.exit_action = QtWidgets.QAction(MainWindow)
        self.exit_action.setObjectName("exit_action")
        self.board_number_auto_inc_action = QtWidgets.QAction(MainWindow)
        self.board_number_auto_inc_action.setCheckable(True)
        self.board_number_auto_inc_action.setObjectName("board_number_auto_inc_action")
        self.actionDebian = QtWidgets.QAction(MainWindow)
        self.actionDebian.setObjectName("actionDebian")
        self.actionRDP_Demo = QtWidgets.QAction(MainWindow)
        self.actionRDP_Demo.setObjectName("actionRDP_Demo")
        self.actionRDP_Demo_2 = QtWidgets.QAction(MainWindow)
        self.actionRDP_Demo_2.setObjectName("actionRDP_Demo_2")
        self.template_2 = QtWidgets.QAction(MainWindow)
        self.template_2.setCheckable(True)
        self.template_2.setChecked(True)
        self.template_2.setObjectName("template_2")
        self.action_save_log = QtWidgets.QAction(MainWindow)
        self.action_save_log.setObjectName("action_save_log")
        self.reset_status_log_action = QtWidgets.QAction(MainWindow)
        self.reset_status_log_action.setObjectName("reset_status_log_action")
        self.about_qt_action = QtWidgets.QAction(MainWindow)
        self.about_qt_action.setObjectName("about_qt_action")
        self.menu.addAction(self.reset_status_log_action)
        self.menu.addSeparator()
        self.menu.addAction(self.exit_action)
        self.settings.addAction(self.board_number_auto_inc_action)
        self.menu_info.addAction(self.about_qt_action)
        self.menubar.addAction(self.menu.menuAction())
        self.menubar.addAction(self.settings.menuAction())
        self.menubar.addAction(self.menu_info.menuAction())

        self.retranslateUi(MainWindow)
        self.current_test_tab_widget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Система запуска тестов"))
        self.test_selected_button.setText(_translate("MainWindow", "Запустить выбранные тесты"))
        self.test_failed_button.setText(_translate("MainWindow", "Запустить упавшие тесты"))
        self.label_3.setText(_translate("MainWindow", "Заводской № платы"))
        self.label_2.setText(_translate("MainWindow", "Чип/платформа"))
        self.current_test_tab_widget.setTabText(self.current_test_tab_widget.indexOf(self.tab), _translate("MainWindow", "Описание теста"))
        self.current_test_tab_widget.setTabText(self.current_test_tab_widget.indexOf(self.tab_2), _translate("MainWindow", "Лог выполнения"))
        self.menu.setTitle(_translate("MainWindow", "Файл"))
        self.settings.setTitle(_translate("MainWindow", "Настройки"))
        self.menu_info.setTitle(_translate("MainWindow", "Справка"))
        self.exit_action.setText(_translate("MainWindow", "Выход"))
        self.board_number_auto_inc_action.setText(_translate("MainWindow", "Автоматически увеличивать зав. № платы"))
        self.actionDebian.setText(_translate("MainWindow", "&Debian"))
        self.actionRDP_Demo.setText(_translate("MainWindow", "&RDP Demo"))
        self.actionRDP_Demo_2.setText(_translate("MainWindow", "RD&P Demo"))
        self.template_2.setText(_translate("MainWindow", "МВ115.&01 (mm7705, процессор-11)"))
        self.action_save_log.setText(_translate("MainWindow", "Сохранить лог"))
        self.reset_status_log_action.setText(_translate("MainWindow", "Сбросить статусы и логи"))
        self.about_qt_action.setText(_translate("MainWindow", "Информация о Qt..."))



import io
from rumboot.edclManager import edclmanager
from xmodem import XMODEM
import inspect

class xferBase():
    connected = False
    chip = None
    maxpayload = 1

    def __init__(self, terminal, params = {}):
        self.term = terminal
        self.params = params
        pass

    def reconnect(self):
        pass
    
    def connect(self, chip):
        self.connected = True
        self.chip = chip
        return True

    def stream_size(self, stream):
        stream.seek(0,2)
        len = stream.tell()
        stream.seek(0)
        return len

    def write8(self, address, data):
        raise Exception("write32 not implemented")

    def read8(self, address):
        raise Exception("read32 not implemented")

    def write16(self, address, data):
        raise Exception("write32 not implemented")

    def read16(self, address):
        raise Exception("read32 not implemented")

    def write32(self, address, data):
        raise Exception("write32 not implemented")

    def read32(self, address):
        raise Exception("read32 not implemented")

    def write64(self, address, data):
        raise Exception("write32 not implemented")

    def read64(self, address):
        raise Exception("read32 not implemented")

    def _write(self, buffer, destaddr, callback = None):
        raise Exception("FATAL: NOT IMPLEMENTED")        

    def _read(self, srcaddr, length, callback = None):
        raise Exception("FATAL: NOT IMPLEMENTED")        

    def read_tlb(self):
        raise Exception("read_tlb not implemented")

    def write_tlb(self, data):
        raise Exception("write_tlb not implemented")

    def write(self, data, destaddr, callback = None):
        if self.connected:
            return self._write(data, destaddr, callback)
        else:
            raise Exception("Transport not connected")

    def read(self, srcaddr, length, callback = None):
        if self.connected:
            return self._read(srcaddr, length, callback)
        else:
            raise Exception("Failed to enable transport")

    def from_file(self, address, fl, callback = None, file_offset = -1, length = -1):
        if type(fl) == str:
            fl = open(fl, "rb")

        if file_offset < 0:
            file_offset = 0

        if length == -1:
            fl.seek(0, 2)
            length = fl.tell() - file_offset

        fl.seek(file_offset)

        chunksize = self.maxpayload * 10
        def wrapcb(total, position, lastwrite):
            if callback:
                callback(length, position, lastwrite)

        while True:
            toread = chunksize
            if toread > length:
                toread = length
            chunk = fl.read(length)
            if len(chunk) == 0:
                break
            self.write(address, chunk, wrapcb)
            address = address + chunksize
            length -= toread
            if length == 0:
                break

    def to_file(self, address, length, fl, callback = None):
        if type(fl) == str:
            fl = open(fl, "wb+")
            needclose = True
        else:
            needclose = False
        if self.maxpayload > 0:
            chunksize = self.maxpayload * 10
        else:
            chunksize = length # Just one big chunk-o-data
        def wrapcb(total, position, lastwrite):
            if callback:
                callback(length, position, lastwrite)
        rd = 0
        while rd < length:
            chunk = self.read(address + rd, chunksize, wrapcb)
            fl.write(chunk)
            rd = rd + chunksize
        if needclose:
            fl.close()

    def close(self):
        pass

class xferXmodem(xferBase):
    maxpayload = -1 # Don't care
    increment = 128
    last_ok = 0
    mode = "xmodem"

    def __init__(self, terminal, params = {}):
        super().__init__(terminal, params)

    def connect(self, chip):
        def getc(size, timeout=10):
            ret = self.term.read(size)
            return ret or None
        def putc(data, timeout=10):
            return self.term.write(data)  # note that this ignores the timeout
        self.modem = XMODEM(getc, putc, mode=self.mode)
        return super().connect(chip)

    def _write(self, destaddr, buffer, callback = None):
        def wrap_callback(total_packets, success_count, error_count):
            if self.last_ok != success_count and success_count != 0:
                if callback:
                    callback(total_packets * self.increment, success_count * self.increment, self.increment)
            self.last_ok = success_count
        if not isinstance(buffer, io.IOBase):
            buffer = io.BytesIO(buffer)

        ln = self.stream_size(buffer)
        return self.modem.send(buffer, retry=128, callback=wrap_callback)

    def _read(self, srcaddr, length, callback = None):
        def wrap_callback(total_packets, success_count, error_count, packet_size):
            if self.last_ok != success_count and success_count != 0:
                if callback:
                    callback(length, success_count * packet_size, packet_size)
            self.last_ok = success_count

        stream = io.BytesIO()
        #HACK: Check if this stuff is merged: https://github.com/tehmaze/xmodem/pull/53
        spec = inspect.getargspec(self.modem.recv)
        if "callback" in spec.args:
            self.modem.recv(stream, crc_mode=0, retry=128, callback=wrap_callback)
        else:
            print("WARN: No progressbar will be shown, because xmodem library is too old")
            print("WARN: Please update (pip install --upgrade xmodem) to see progressbar during readdout")
            self.modem.recv(stream, crc_mode=0, retry=128)
        return stream.getvalue()

class xferXmodem1k(xferXmodem):
    mode = "xmodem1k"
    increment = 1024

class xferEdcl(xferBase):
    edcl = None
    def __init__(self, terminal, params):
        super().__init__(terminal, params)

    def connect(self, chip):
        if not self.edcl:
            self.edcl = edclmanager()
            self.edcl.force_static_arp = self.params["force_static_arp"]
            if not self.edcl.connect(chip, self.params):
                print("ERROR: Failed to establish edcl connection")
                return False
            self.maxpayload = self.edcl.maxpayload
        return super().connect(chip)
        
    def _write(self, destaddr, buffer, callback = None):
        return self.edcl.write(destaddr, buffer, callback)

    def _read(self, srcaddr, length, callback = None):
        return self.edcl.read(srcaddr, length, callback)

    def reconnect(self):
        if self.edcl != None:
            self.edcl.reconnect()

    def write32(self, address, data):
        return self.edcl.write32(address, data)

    def read32(self, address):
        return self.edcl.read32(address)

# Params is a dict:
#   default: "xmodem"
#   edcl_ip: "192.168.0.1"
#   edcl_mac: "0:0:5e:0:0:0"
#   edcl_timeout: 7.0
#   force_static_arp: True/False
#
class xferManager():
    xfers = {}
    xfer = None
    how = "xmodem"
    params = None

    def __init__(self, terminal, params = {}):
        self.term = terminal
        self.params = params
        if params == None:
            params = {}
        if not "default" in params:
            params["default"] = "xmodem"
        if not "force_static_arp" in params:
            params["force_static_arp"] = False

        self.xfers["xmodem"] = xferXmodem1k(terminal, params)
        self.xfers["xmodem-128"] = xferXmodem(terminal, params)
        self.xfers["edcl"] = xferEdcl(terminal, params)
        #self.xfers["ubootmx"] = xferUbooMX(terminal, params)
        self.how = params["default"]
        self.xfer = self.xfers[self.how]

    def setChip(self, chip):
        self.chip = chip
   
    def selectTransport(self, how):
        self.how = how
        if not how in self.xfers:
            raise Exception("Unknown xfer method: %s" % how)
        self.xfer = self.xfers[how]

    #Forward API calls to selected transport
    def __getattr__(self, name):
        return getattr(self.xfer, name)

    #TODO: This logic needs heavy refactor
    def push(self, address):
        binary = self.term.next_binary()
        self.send(address, binary)
        return True

    def send(self, destaddr, stream, desc='Sending stream'):
        needclose = False
        if type(stream) == str:
            stream = open(stream, "rb")
            needclose = True
        elif not isinstance(stream, io.IOBase):
            stream = io.BytesIO(stream)
            needclose = True

        def prg(total_bytes, position, increment):
            self.term.progress_update(total_bytes, position, increment)

        if not self.xfer.connect(self.chip):
            raise Exception("Failed to connect transport")

        self.term.progress_start(desc, self.stream_size(stream))
        self.from_file(destaddr, stream, file_offset = 0, callback=prg)
        self.term.progress_end()
        if needclose:
            stream.close()
        return True

    def recv(self, stream, srcaddr, total, desc="Receiving data"):
        if not self.xfer.connect(self.chip):
            raise Exception("Failed to connect transport")

        return self.to_file(srcaddr, total, stream)



import fnmatch


def update_suitable(test_registry, test_context):
    def update_one_test(test_desc):
        test_desc.suitable = False
        if test_context.env["tests"]["enabled"]:
            for enabled_pattern in test_context.env["tests"]["enabled"]:
                if fnmatch.fnmatch(test_desc.full_name, enabled_pattern):
                    break
            else:
                return
        if test_context.env["tests"]["disabled"]:
            for disabled_pattern in test_context.env["tests"]["disabled"]:
                if fnmatch.fnmatch(test_desc.full_name, disabled_pattern):
                    return
        test_desc.suitable = test_desc.test_class.suitable(test_context.env, test_desc.params)
    test_registry.test_iteration(update_one_test)



import os
import time
import usb.core
import pygpiotools
import usb.util
from parse import parse
from rumboot.resetseq.resetSeqBase import base
from serial import Serial

class mdb(base):
    name = "Malina Debug Bridge"
    swap   = False
    supported = ["POWER", "RESET", "HOST", "EDCL_LOCK"]

    def __init__(self, terminal, opts):
        port = os.path.realpath(terminal.ser._port)
        self.ctlport = opts["mdb_ctl_port"]
        if self.ctlport is None:
            id = parse("/dev/ttyACM{:d}", port)[0]
            id += 1
            self.ctlport = f"/dev/ttyACM{id}"
        self.serial = Serial(self.ctlport, 115200)        
        super().__init__(terminal, opts)
        self["EDCL_LOCK"] = 0
        self["HOST"] = 0

    def __setitem__(self, key, value):
        super().__setitem__(key, value)

        if "HOST" in self._states and "EDCL_LOCK" in self._states:
            self.serial.write(f"bootm {1 - self._states['HOST']} {self._states['EDCL_LOCK']}\r\n".encode())
        if "RESET" in self._states:
            self.serial.write(f"rst {1 - self._states['RESET']}\r\n".encode())



    def get_options(self):
        return {
                "mdb-ctl-port" : {
                    "help" : "Malina Debug Bridge port",
                    "default" : None,
                },
            }


import sys
import time
from rumboot.resetseq.resetSeqBase import base
import pygpiotools
import os

#   MT125.05. Shift register on CBUS pins
class mt12505(base):
    name = "MT125.05 (FT232RL)"
    _bits = [0, 0, 0, 0, 0, 0, 0, 0]

    # CB0 : USB_DIN / USB_DOUT
    # CB1 : USB_LOAD
    # CB2 : USB_OE
    # CB3 : USB_CLK 
    # CB4 : LED 

    # Default mapping (from basis)
    # Normally, we should load mapping from per-chip yaml
    mapping = { }

    def __reg_write_bit(self, bit):
        if bit>0:
            v = 0
        else:
            v = 1
        self.sp.cbus_write(v)    
        self.sp.cbus_write(v | (1<<3))
        self.sp.cbus_write(v)
        self.sp.cbus_write(0)

    def __write_reg(self, values):
        import traceback

        self.sp.cbus_write(0)
        for v in reversed(values):
            self.__reg_write_bit(v)
        self.sp.cbus_write(0)
        self.sp.cbus_write(1<<2)
        self.sp.cbus_write(0)

    def __to_mt12505_devices(self, devs):
        ret = []
        for d in devs:
            if d[1].find("MT12505") != -1:
                ret.append(d)
        return ret


    def set_chip(self, chip):
        if hasattr(chip, "mt12505"):
            self._bits = chip.mt12505["default"]
            self.mapping = chip.mt12505["mapping"]
            self.supported = []
            for k,v in self.mapping.items():
                self.supported.append(k)
            print(f"MT12505: Loaded mapping for '{chip.name}'. Defaults: {chip.mt12505['default']}")
            print(f"MT12505: Supported controls: {self.supported}")
        return super().set_chip(chip)

    def __init__(self, terminal, opts):
        # Lazy-init FT232. 
        # On windows systems, if D2XX drivers are not installed
        try:
            import ft232
        except:
            print("ERROR: Failed to load pyft232")
            print("ERROR: Is D2XX/libftdi installed?")
            sys.exit(1)
        # HACK: On some linux systems (debian buster) we can't free libftdi context
        # HACK: or things will segfault
        # HACK: This hack causes intentional memory leak, but prevents segfault
        def dummy(x):
            pass
        try: 
            ft232.libftdi.ftdi.ftdi_free = dummy
        except:
            pass

        serial = opts["mt12505_serial"]
        if opts["mt12505_serial"] == None:
            devices = ft232.list_devices()
            devices = self.__to_mt12505_devices(devices)
            if len(devices) == 0:
                raise Exception("ERROR: No MT12505 devices found") 

            if len(devices) == 1:
                serial = devices[0][0]
            else:
                print("WARNING: Multiple MT12505 devices found, please specify serial")
                for d in devices:
                    print(f"Serial: {d[0]} Description: {d[1]}")
                sys.exit(1)

        self.sp = ft232.Ft232(serial, baudrate=115200)
        self.sp.cbus_setup(mask=0xf, init=0xf)
        return super().__init__(terminal, opts)

    def __setitem__(self, key, value):
        super().__setitem__(key, value)
        key = self.mapping[key]
        if key >= 0: #Silently ignore dummy items
            self._bits[key] = value
        self.__write_reg(self._bits)

    #No reset/power inversions on these boards
    def reset(self):
        self["RESET"] = 0
        self["POWER"] = 0
        time.sleep(1)
        self["RESET"] = 1
        self["POWER"] = 1

    def get_options(self):
        return {
            "mt12505-serial" : {
                "help"     : "FT232RL serial number to use",
                "default"  : None,
                "required" : False
            }
        }




import xml.dom.minidom 
import os
from rumboot.testing.test_desc import *


class JUintReport:

    def __init__(self, log_file_path):
        self._log_file_path = log_file_path
        self._tests = 0
        self._failures = 0
        self._root = xml.dom.minidom.Document()
        self._testsuites = self._root.createElement("testsuites")
        self._root.appendChild(self._testsuites)
        self._testsuite = self._root.createElement("testsuite")
        self._testsuite.setAttribute("name", "Testing result") # ???
        self._testsuites.appendChild(self._testsuite)

    def add_test_result(self, test_desc):
        self._tests += 1
        testcase = self._root.createElement("testcase")
        testcase.setAttribute("name", test_desc.full_name)
        if test_desc.status == TEST_STATUS_NOT_EXECUTED:
            skipped = self._root.createElement("skipped")
            skipped.setAttribute("message", "Skipped")
            testcase.appendChild(skipped)
        elif test_desc.status == TEST_STATUS_FAULT:
            self._failures += 1
            failure = self._root.createElement("failure")
            failure.setAttribute("message", "Failed")
            testcase.appendChild(failure)
        if test_desc.status != TEST_STATUS_NOT_EXECUTED:
            std_out = self._root.createElement("system-out")
            text_value = self._root.createTextNode(test_desc.log_text)
            std_out.appendChild(text_value)
            testcase.appendChild(std_out)
        self._testsuite.appendChild(testcase)

    def flush(self):
        self._testsuite.setAttribute("tests", str(self._tests))
        self._testsuite.setAttribute("errors", "0")
        self._testsuite.setAttribute("failures", str(self._failures))

        file_path = os.path.join(self._log_file_path)
        with open(file_path, "w") as file:
            self._root.writexml(file, addindent="  ", newl="\n", encoding="UTF-8")



import os
import sys
import argparse
import atexit
import threading
import time
import copy
import io
import random
import humanfriendly
import yaml

from parse import *

from rumboot.chipDb import ChipDb
from rumboot.ImageFormatDb import ImageFormatDb
from rumboot.resetSeq import ResetSeqFactory
from rumboot.cmdline import arghelper
from rumboot.terminal import terminal
import rumboot_xrun
import rumboot

from .base import PartitionBase, FlashDeviceBase

from rumboot.chipDb import ChipDb
from rumboot.ImageFormatDb import ImageFormatDb
from rumboot.resetSeq import ResetSeqFactory
from rumboot.cmdline import arghelper
from rumboot.terminal import terminal

import os
import argparse
import rumboot_packimage
from parse import *
import rumboot
from inspect import isclass

from rumboot.classLoader import classLoader

class FlashAlgoFactory(classLoader):
    def __init__(self, path, protocol):
        super().__init__(path)
        self.protocol = protocol

    def __getitem__(self, key):
        for k,v in self.classes.items():
            if issubclass(v,PartitionBase) and issubclass(v,FlashDeviceBase):
                rt = parse(v.device, key)
                if rt and self.protocol == v.protocol:
                    return v

def execute_runlist(term, spl, runlist):
    def prg(total_bytes, position, increment):
        term.progress_update(total_bytes, position, increment)

    for item in runlist:
        partition = item["partition"]
        mem = partition.name
        fl = item['file']

        if item["action"] == "write":
            fl = fl.replace("@SPL", spl)
            fl = open(fl, "rb")
            size = partition.stream_size(fl)
            if size > partition.size:
                print("WARN: File too big and will be truncated")
                size = partition.size

            if not partition.skip_erase_before_write:
                term.progress_start(f"Erasing {mem}", size)
                partition.erase(0, partition.size, callback=prg)
                term.progress_end()

            term.progress_start(f"Writing {mem}", size)
            partition.write(fl, 0, size, callback=prg)
            term.progress_end()
            fl.close()

        if item["action"] == "erase":
            term.progress_start(f"Erasing {mem}", partition.size)
            partition.erase(0, partition.size, callback=prg)
            term.progress_end()

        if item["action"] == "read":
            fl = item["file"]
            fl = open(fl, "wb")
            term.progress_start(f"Reading {mem}", partition.size)
            partition.read(fl, 0, partition.size, callback=prg)
            term.progress_end()
            fl.close()
    


def parse_firmware_file(file, flasher):
    cnf = yaml.load(file, Loader=yaml.FullLoader)
    ret = []
    for name,data in cnf["partitions"].items():
        if type(data["size"]) == str:
            size = humanfriendly.parse_size(data["size"])
        else:
            size = data["size"]            

        fl = None
        offset = None
        if "offset" in data:
            offset =  data["offset"]
        if "file" in data:
            fl =  data["file"]
            action = "write"
        if "action" in data:
            action = data["action"]
        part = flasher.add_partition(name, offset, size)
        item = {
            "partition" : part,
            "action"    : action,
            "file"      : fl
        }
        ret.append(item)

    for key, value in cnf["environment"].items():
        flasher.env(key, value)

    return ret

def rumboot_start_flashing(partmap=None):
    resets  = ResetSeqFactory("rumboot.resetseq")
    formats = ImageFormatDb("rumboot.images")
    chips   = ChipDb("rumboot.chips")

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description="rumboot-xflash {} - RumBoot firmware updater tool\n".format(rumboot.__version__) +
                                    rumboot.__copyright__)
    helper = arghelper();                                
    helper.add_file_handling_opts(parser, True)
    helper.add_terminal_opts(parser)
    parser.add_argument("-v", "--verbose", 
                        default=False,
                        action='store_true',
                        help="Print serial debug messages during update"),
    parser.add_argument("-m", "--memory",
                        help="Memory program. Help for a list of memories",
                        metavar=('memory'),
                        default="help")
    parser.add_argument("-z", "--spl-path",
                        help="Path for SPL writers (Debug only)",
                        type=str,
                        required=False)
    parser.add_argument("--no-spl",
                        help="Do not upload spl, assume it boots on it's own (Debug only)",
                        action="store_true",
                        default=False,
                        required=False)
    parser.add_argument("-o", "--offset",
                        help="Memory offset for read/write operations",
                        type=int,
                        default=0,
                        required=False)
    parser.add_argument("-L", "--length",
                        help="How many bytes to read/write (defaults to whole file/flash)",
                        type=int,
                        default=-1,
                        required=False)
    parser.add_argument("-R", "--read",
                        help="Read flash to file",
                        action='store_true',
                        required=False)
    parser.add_argument("-W", "--write",
                        help="Write flash from file",
                        default=False,
                        action='store_true',
                        required=False)
    parser.add_argument("-E", "--erase",
                        help="Erase flash",
                        default=False,
                        action='store_true',
                        required=False)
    parser.add_argument("-F", "--firmware-file",
                        help="Write firmware from configuration file",
                        type=argparse.FileType("r"),
                        default=None,
                        required=False)
    parser.add_argument("-U", "--upload-baudrate",
                        help="Change baudrate for uploads",
                        default=0,
                        type=int,
                        required=False)

    helper.add_resetseq_options(parser, resets)

    opts = parser.parse_args()

    chip = helper.detect_chip_type(opts, chips, formats)
    if not chip:
        return 1

    mem = opts.memory
    if mem == "help":
        print(f"Available memories for chip '{chip.name}'")
        for mem,cfg in chip.memories.items():
            print("%16s: %s" % (mem, cfg["comment"]))
        return 1

    chip, term, reset = helper.create_core_stuff_from_options(opts)

    if opts.read and opts.write:
        print("Please select either --read or --write, not both")
        return 1

    #Open files, rebuild if needed
    #opts.file[0], dumps = helper.process_files(opts.file[0], False)

    spl_path = os.path.dirname(__file__) + "/spl-tools/"

    try:
        config = chip.memories[mem]
        mem = config["device"]
    except:
        print("ERROR: Target chip (%s) doesn't have memory '%s'. Run with -m help for a list" % (chip.name, mem))
        return 1

    if opts.spl_path != None:
        spl_path = opts.spl_path
        print("SPL                              %s" % (spl_path + chip.memories[mem]))

    spl = spl_path + config["spl"]

    term.verbose = opts.verbose
    if not opts.no_spl:
        term.add_binaries(spl)
    if opts.edcl and chip.edcl != None:
        term.xfer.selectTransport("edcl")

    try:
        reset.resetToHost()
    except:
        print("WARN: Reset method doesn't support HOST mode switching")
        print("WARN: If things don't work - check jumpers!")
        reset.reset()
        
    term.loop(break_after_uploads=True)

    flashers = FlashAlgoFactory("rumboot.flashing", config["protocol"])
    flasher = flashers[mem](term, config)
    # ------------------------
    offset = opts.offset
    length = -1

    if "offset" in config:
        offset += config["offset"]
    if "size" in config:
        flasher.size = config["size"]

    flasher.dump()

    if opts.file and len(opts.file) > 1:
        print("ERROR: Only one file may be specified") 
        return 1

    if opts.file and len(opts.file) == 0:
        return 1

    if opts.upload_baudrate == 0 and hasattr(chip, "flashbaudrate"):
         opts.upload_baudrate = chip.flashbaudrate

    if opts.upload_baudrate > 0:
        print(f"WARN: Changing baudrate to {opts.upload_baudrate} bps")
        print( "WARN: If everything freezes - try a different setting via -U / --upload-baudrate")
        flasher.switchbaud(opts.upload_baudrate)

    term.xfer.connect(chip)

    runlist = []
    if opts.firmware_file is None:
        part = flasher.add_partition(mem, opts.offset, opts.length)
        if opts.write:
            action = {
                "partition" : part,
                "action"    : "write",
                "file"      : opts.file[0][0]
            }
            runlist.append(action)

        if opts.read:
            action = {
                "partition" : part,
                "action"    : "read",
                "file"      : opts.file[0][0]
            }
            runlist.append(action)

        if opts.erase:
            action = {
                "partition" : part,
                "action"    : "erase",
                "file"      : None
            }
            runlist.append(action)
    else:
        runlist = parse_firmware_file(opts.firmware_file, flasher)

    execute_runlist(term, spl, runlist)

    if opts.firmware_file or opts.write:
        print("Saving environment and partition information")
        flasher.save_partitions()
        flasher.saveenv()

    return 1




import os
import argparse
from rumboot.cmdline import arghelper
from rumboot.resetSeq import ResetSeqFactory
from rumboot.chipDb import ChipDb
from rumboot.testing.yaml import *


DEFAULT_ENV_FILE_NAME = "env.yaml"


class TestContext:

    def __init__(self):
        self.opts = None
        self.env = {}
        self.resets = ResetSeqFactory("rumboot.resetseq")
        self.chips = ChipDb("rumboot.chips")
        self.chip = None

    def process_cmdline(self):
        parser = argparse.ArgumentParser(prog="<rumboot test system>", description="Processing all tests")
        parser.add_argument("-C", "--directory", dest="root_path", help="test root directory", required=False)
        parser.add_argument("--env", dest="env_path", help="environment yaml file", required=False)
        parser.add_argument("--report", dest="report_file_path", help="JUnit report file", required=False)
        parser.add_argument("--gui", dest="gui", help="start GUI mode", action="store_true", default=False)
        parser.add_argument("--tests-enabled", dest="tests_enabled", help="enabled test templates", required=False, nargs="*")
        parser.add_argument("--tests-disabled", dest="tests_disabled", help="disabled test templates", required=False, nargs="*")

        helper = arghelper()
        helper.add_terminal_opts(parser)
        helper.add_resetseq_options(parser, self.resets)
        helper.add_file_handling_opts(parser) # ??? file

        self.opts = parser.parse_args()

    def load_environment(self):
        self.env = {}
        if self.opts.env_path != None:
            self.env = load_yaml_from_file(self.opts.env_path)
        else:
            if os.path.isfile(DEFAULT_ENV_FILE_NAME):
                self.env = load_yaml_from_file(DEFAULT_ENV_FILE_NAME)

    def setup_environment(self):
        self.env["chip"] = self.env.get("chip", {})
        chip_id_or_name = self.env["chip"].get("name", None)
        if self.opts.chip_id:
            chip_id_or_name = self.opts.chip_id[0]
        self.chip = self.chips[chip_id_or_name]
        if self.chip == None:
            raise Exception("Failed to detect chip type")

        self.env["chip"]["name"] = self.chip.name

        self.env["connection"] = self.env.get("connection", {})
        self.env["connection"]["port"] = self.env["connection"].get("port", None)
        if self.opts.port:
            self.env["connection"]["port"] = self.opts.port
        if not self.env["connection"]["port"]:
            raise Exception("Serial port is not defined")

        self.env["connection"]["baud"] = self.env["connection"].get("baud", self.chip.baudrate)
        if self.opts.baud:
            self.env["connection"]["baud"] = self.opts.baud[0]
        if not self.env["connection"]["baud"]:
            raise Exception("Serial port baudrate is not defined")

        self.env["connection"]["transport"] = self.env["connection"].get("transport", "xmodem")
        if self.opts.edcl:
            self.env["connection"]["transport"] = "edcl"
        self.env["connection"]["force_static_arp"] = self.env["connection"].get("force_static_arp", False)
        if self.opts.force_static_arp:
            self.env["connection"]["force_static_arp"] = True
        self.env["connection"]["edcl_ip"] = self.env["connection"].get("edcl_ip", None)
        if self.opts.edcl_ip:
            self.env["connection"]["edcl_ip"] = self.opts.edcl_ip
        self.env["connection"]["edcl_mac"] = self.env["connection"].get("edcl_mac", None)
        if self.opts.edcl_mac:
            self.env["connection"]["edcl_mac"] = self.opts.edcl_mac
        self.env["connection"]["edcl_timeout"] = self.env["connection"].get("edcl_timeout", None)
        if self.opts.edcl_timeout:
            self.env["connection"]["edcl_timeout"] = self.opts.edcl_timeout

        self.env["reset"] = self.env.get("reset", {})
        self.env["reset"]["name"] = self.env["reset"].get("name", "none")
        if self.opts.reset != "none":
            self.env["reset"]["name"] = self.opts.reset[0]
        opts_vars = vars(self.opts)
        for sequence in self.resets.classes.values():
            if hasattr(sequence, "get_options"):
                options = sequence.get_options(sequence)
                for key in options.keys():
                    right_key = key.replace("-", "_")
                    if opts_vars[right_key] or not right_key in self.env["reset"]:
                        self.env["reset"][right_key] = opts_vars[right_key]
        self.env["reset"]["port"] = self.env["connection"]["port"]

        root_path = self.env.get("root_path")
        if self.opts.root_path:
            root_path = self.opts.root_path
        if not root_path:
            root_path = os.path.curdir
        self.env["root_path"] = os.path.abspath(root_path)

        self.env["report_file_path"] = self.env.get("report_file_path", None)
        if self.opts.report_file_path:
            self.env["report_file_path"] = self.opts.report_file_path
        if self.env["report_file_path"]:
            self.env["report_file_path"] = os.path.abspath(self.env["report_file_path"])

        self.env["gui"] = self.opts.gui

        self.env["tests"] = self.env.get("tests", {})
        self.env["tests"]["enabled"] = self.env["tests"].get("enabled", [])
        if self.opts.tests_enabled:
            self.env["tests"]["enabled"] = self.opts.tests_enabled
        self.env["tests"]["disabled"] = self.env["tests"].get("disabled", [])
        if self.opts.tests_disabled:
            self.env["tests"]["disabled"] = self.opts.tests_disabled



import serial
import sys
from xmodem import XMODEM
import os
import fcntl
from parse import parse
import time
import io
from tqdm import tqdm
from rumboot.OpFactory import OpFactory
import threading
import serial
import serial.rfc2217
import socket
import select
import signal
import tempfile
import subprocess

class redirector(threading.Thread):
    alive = False
    fatal = False
    callback = None
    started = 0
    max_connected_time = -1

    def configure(self, serial, socket, timeout):
        self.serial = serial
        self.socket = socket
        self.max_connected_time = timeout

    def set_callback(self, cb):
        self.callback = cb

    def cleanup(self, fatal):
        self.socket.close()
        self.fatal = fatal
        self.alive = False
        if not fatal:
            print("Client disconnected")
        else:
            print("Something bad happened. Stopping daemon")
        if self.callback != None:
            print("calling", fatal)
            self.callback(fatal)

    def check_timeout(self):
        if self.max_connected_time == -1:
            return False # No timeout
        if time.monotonic() - self.started > self.max_connected_time:
            self.socket.close()
            return True
        return False

    def readSerial(self):
        if isinstance(self.serial, serial.Serial):
            return bytearray(self.serial.read(self.serial.inWaiting()))
        else:
            raise Exception("Not pyserial!")

    def writeSerial(self, data):
        if isinstance(self.serial, serial.Serial):
            return self.serial.write(data)
        else:
            raise Exception("Not pyserial!")

    def nodelay(self, sock):
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
        sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, 1)
        sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, 1)
        sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, 2)
        sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)

    def nonblock(self, fd):
        fl = fcntl.fcntl(fd, fcntl.F_GETFL)
        fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)

    def loop_once(self):
        fromserial = bytearray(b"")
        fromsocket = bytearray(b"")

        ready_to_read, ready_to_write, in_error = select.select(
            [self.socket, self.serial.fileno()],
            [self.socket, self.serial.fileno()],
            [self.socket, self.serial.fileno()],
            1)
        for sock in ready_to_read:       
            if sock == self.serial.fileno():
                fromserial = fromserial + self.readSerial()
                if self.socket in ready_to_write:
                    sent = self.socket.send(fromserial)
                    del fromserial[0:sent]
            if sock == self.socket:
                tmp = self.socket.recv(1024)
                if len(tmp) == 0:
                    self.cleanup(False)
                    return False
                fromsocket = fromsocket + bytearray(tmp)
                if self.serial.fileno() in ready_to_write:
                    ret = self.writeSerial(fromsocket)
                    del fromsocket[0:ret]
        for sock in in_error:
            if sock == self.serial.fileno():
                print("Something bad with serial port")
                self.cleanup(True)
                return False
            if sock == self.socket:
                print("Disconnect?")
                self.cleanup(False)               
                return False
        return True

    def run(self):
        self.alive = True
        self.fatal = False
        self.socket.setblocking(0)
        self.nodelay(self.socket)
        self.started = time.monotonic()
        try:
            while self.loop_once():
                if self.check_timeout():
                    print("Time's up. Kicking client...")
                    self.cleanup(False)
                    return

        except BrokenPipeError:
            self.cleanup(False)
            return
        except ConnectionResetError:
            self.cleanup(False)
            return
        except OSError:
            self.socket.close()
            self.cleanup(True)
            raise
        except Exception:
            self.cleanup(False)
            raise

class appredirector(redirector):
    welcome = b'''
    RC Module's          __                __
   _______  ______ ___  / /_  ____  ____  / /_
  / ___/ / / / __ `__ \/ __ \/ __ \/ __ \/ __/
 / /  / /_/ / / / / / / /_/ / /_/ / /_/ / /_
/_/   \__,_/_/ /_/ /_/_.___/\____/\____/\__/

boot: Native Debug Environment
boot: host: Hit 'X' for X-Modem upload


'''

    def cleanup(self, fatal):
        #TODO: Kill the app
        print("Cleaning up corpses...")
        self.pipe.kill()
        os.unlink(self.tempfile)
        return super().cleanup(fatal)

    def run(self):
        self.nodelay(self.socket)
        self._buf = b""
        def getc(size, timeout=10):
            ready = select.select([self.socket], [], [], timeout)
            ret = None
            if ready[0]:
                ret = self.socket.recv(size)
            return ret

        def putc(data, timeout=10):
            ret = self.socket.sendall(data, socket.MSG_DONTWAIT)
            return len(data)

        self.modem = XMODEM(getc, putc, mode="xmodem1k")
        self.socket.sendall(self.welcome, socket.MSG_DONTWAIT)
        while True:
            data = self.socket.recv(1)
            print(data)
            if data == b'X':
                break

        #Workaround: We need three Cs: 'CC' is required for a sync, one for actual xmodem transfer
        ret = self.socket.sendall(b"CC", socket.MSG_DONTWAIT)
        tmp = tempfile.NamedTemporaryFile(mode="wb+", prefix="rumboot_daemon_temp_", delete=False)
        try:
            self.modem.recv(tmp)
        except Exception as e:
            self.socket.sendall(str(e), socket.MSG_DONTWAIT)
            return 0
        tmp.close()
        os.chmod(tmp.name, 755)
        print('--- starting binary ---')
        self.pipe = subprocess.Popen([tmp.name], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
        self.nonblock(self.pipe.stdout.fileno())
        self.tempfile = tmp.name
        return super().run()

    def readSerial(self):
        try:
            ret = self.pipe.stdout.read()
            return ret
        except Exception as e:
            print(e);
            return b""

    def writeSerial(self, data):
        self.pipe.stdin.write(data)
        self.pipe.stdin.flush()

    fromserial = bytearray(b"")
    fromsocket = bytearray(b"")

    def loop_once(self):
        ready_to_read, ready_to_write, in_error = select.select(
            [self.socket, self.pipe.stdout],
            [self.socket, self.pipe.stdin],
            [self.socket],
            1)

        for sock in ready_to_read:
            if sock == self.pipe.stdout:
                self.fromserial = self.fromserial + self.readSerial()
                if self.socket in ready_to_write:
                    sent = self.socket.send(self.fromserial)
                    del self.fromserial[0:sent]

            if sock == self.socket:
                tmp = self.socket.recv(1024)
                if len(tmp) == 0:
                    self.cleanup(False)
                    return False
                self.fromsocket = self.fromsocket + bytearray(tmp)
                if self.pipe.stdin in ready_to_write:
                    ret = self.writeSerial(self.fromsocket)
                    del self.fromsocket[0:ret]

        for sock in ready_to_write:
            if sock == self.pipe.stdin and len(self.fromsocket) > 0:
                ret = self.writeSerial(self.fromsocket)
                del self.fromsocket[0:ret]

            if self.socket in ready_to_write and len(self.fromserial) > 0:
                try:
                    sent = self.socket.send(self.fromserial)
                    del self.fromserial[0:sent]
                except BlockingIOError:
                    pass

        for sock in in_error:
            fromserial = bytearray(b"")
            fromsocket = bytearray(b"")
            if sock == self.serial.fileno():
                print("Something bad with serial port")
                self.cleanup(True)
                return False
            if sock == self.socket:
                print("Disconnect?")
                self.cleanup(False)               
                return False

        if self.pipe.poll() is not None and len(self.fromserial) == 0:
            self.socket.sendall(f"boot: host: Back in rom, code {self.pipe.poll()}\n".encode())
            self.cleanup(False)
            return False

        return True
class server:
    client_queue = [ ]
    worker = None
    #Graveyard of zombies
    graveyard = []
    pid = os.getpid()
    binaries = []

    def __init__(self, terminal, tcplisten, timeout=-1, isnativedebug=False):
        self.serial = terminal.serial()
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        addr, port = tcplisten.split(":")
        self.sock.bind((addr, int(port)))
        self.term = terminal
        self.max_connected_time = timeout
        self.isnativedebug = isnativedebug
        terminal.chip.hacks["skipsync"] = True

    def set_reset_seq(self, rst):
        self.rst = rst

    def preload_binaries(self, files):
        self.binaries = files

    def serve_once(self):
        def the_callback(fatal = False):
            if not fatal:
                self.serve_once()
            else:
                print("Interrupting main thread")
                os.kill(self.pid, signal.SIGINT)

        if self.worker != None:
            if self.worker.alive:
                return #We're busy here
            else:
                #Put our dead worker to the graveyard
                self.graveyard = self.graveyard + [ self.worker ]

        try:
            client = self.client_queue.pop(0)
            # Reset if using a nested damon
            self.term.reopen()
            self.serial = self.term.serial()

            try:
                self.rst.resetToHost()
            except:
                self.rst.reset()

            if self.binaries:
                text = b"U\nrumboot-daemon: Preloading your board board...\n\n\n"
                client["connection"].sendall(text)

                self.term.add_binaries(self.binaries)
                self.term.loop(break_after_uploads=True)

            if not self.isnativedebug:
                self.worker = redirector()
            else:
                self.worker = appredirector()

            self.worker.configure(self.serial, client["connection"], self.max_connected_time)
            self.worker.set_callback(the_callback)

            # We can't do it, if we're working remotely
            # Or somebody resets us manually
#            if type(self.rst).__name__ != "base":
#                self.serial.reset_input_buffer()
#                self.serial.reset_output_buffer()
            
            self.worker.start()
            print("Now serving client: ", client["dns"])
        except(IndexError):
            self.worker = None
            try:
                self.rst.power(0) # Power off board
            except:
                pass #No power control, no problem. It's optional

    def queue_client(self, connection, client_address):
        try:
            dns = socket.gethostbyaddr(client_address[0])
        except:
            dns = "<unknown>"
        print("Incoming connection: ", dns)
        client = { }
        client["connection"] = connection
        client["address"] = client_address
        client["dns"] = dns
        self.client_queue.append(client)
        pos = len(self.client_queue)
        if self.worker != None:
            pos = pos + 1

        text = b"U\nrumboot-daemon: You are client number %d in queue, please stand by\n\n\n" % pos
        connection.sendall(text)

        if self.worker == None:
            self.serve_once()


    def kill_zombies(self):
        #Kill all zombies.
        for z in self.graveyard:
            z.join()
        self.graveyard = [ ]                    

    def cleanup(self):
        if self.worker:
            self.worker.join()
        self.worker.socket.close()
        self.sock.close()

    def loop(self):
        try:
            self.rst.power(0) # Power off board, if can
        except:
            pass
        try:
            self.sock.listen()
            self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
            self.sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPIDLE, 1)
            self.sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPINTVL, 1)
            self.sock.setsockopt(socket.SOL_TCP, socket.TCP_KEEPCNT, 2)

            while True:
                print('waiting for a connection')
                connection, client_address = self.sock.accept()
                self.queue_client(connection, client_address)
                self.kill_zombies()
        except:
            self.cleanup()
        finally:
            self.cleanup()



import os
from time import sleep
import socket
from rumboot.resetseq.resetSeqBase import base

class apc(base):
    name = "APC Switched Racked PDU"
    host = "192.168.10.2"
    port = 23
    user = "whiteblade"
    passwd = "ash2wert"
    outlet = 8

    def __init__(self, terminal, opts):
        self.passwd = opts["apc_pass"]
        self.host   = opts["apc_host"]
        self.user   = opts["apc_user"]
        self.outlet = opts["apc_outlet"]
        super().__init__(terminal,opts)

    def __expect(self, f, inw, outw):
        failures = 0
        while True:
            if failures > 30:
                raise Exception("APC didn't reply in time")
            tmp = f.readline()
            if len(tmp) == 0:
                sleep(0.1)
                failures += 1
                pass
            if inw in tmp:
                f.write(outw + "\r")
                f.flush()
                return

    def __power(self, on):
            s = socket.create_connection((self.host, self.port), timeout=3)
            s.setblocking(0)
            if on:
                on = 1
            else:
                on = 2
            f = s.makefile('rw', encoding="ascii", errors="replace", newline='\r')
            self.__expect(f, "User Name :", self.user)
            self.__expect(f, "Password  :", self.passwd)
            self.__expect(f, ">", "1")
            self.__expect(f, ">", "2")
            self.__expect(f, ">", "1")
            self.__expect(f, ">", str(self.outlet))
            self.__expect(f, ">", str(on))
            self.__expect(f, "Enter 'YES' to continue or <ENTER> to cancel", "YES")
            self.__expect(f, "Press <ENTER> to continue...", "")
            s.close()
            sleep(3) #Power bricks have huge caps. Let power stabilize

    def power(self, on):
        #Sometimes we can connect, but get back nothing
        retries = 4
        while retries:
            try:
                self.__power(on)
                return
            except Exception as e:
                print(f"APC: APC Operation failed: {e}")
                retries = retries - 1
                continue
        raise Exception("APC: Too many APC operations failed, giving up")

    #We always rely on power-on reset
    def __setitem__(self, key, value):
        super().__setitem__(key, value)

        if key == "POWER":
            self.power(value)

        if key == "RESET":
            pass

    def get_options(self):
        return {
                "apc-host" : {
                    "help" : "APC IP Address/hostname",
                    "default" : "192.168.10.2",
                    "required": False
                },
                "apc-user" : {
                    "help" : "APC Username",
                    "default" : "whiteblade",
                    "required": False
                },
                "apc-pass" : {
                    "help" : "APC Password",
                    "default" : "ash2wert",
                    "required": False
                },
                "apc-outlet" : {
                    "help" : "APC Outlet Number",
                    "default" : 8,
                    "required": False
                }
            }


from rumboot.cmdline import arghelper
from rumboot.resetSeq import ResetSeqFactory
import os
import sys
import argparse
import rumboot_xrun
import rumboot
from parse import *

def cli():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description="rumboot-xrun {} - RumBoot X-Modem execution tool\n".format(rumboot.__version__) +
                                    rumboot.__copyright__)
    helper = arghelper()
    resets  = ResetSeqFactory("rumboot.resetseq")
    helper.add_file_handling_opts(parser, True)
    helper.add_terminal_opts(parser)
    helper.add_resetseq_options(parser, resets)
    plus = parser.add_argument_group("Plusargs parser options",
        """
        rumboot-xrun can parse plusargs (similar to verilog simulator)
        and use them for runtime file uploads. This option is intended
        to be used for
        """)
    plus.add_argument('-A', '--plusargs', action="append", nargs='*')
    parser.add_argument("-R", "--rebuild",
                        help="Attempt to rebuild/update target before uploading",
                        action="store_true",
                        default=False,
                        required=False)

    parser.add_argument("-I", '--stdin',
                        help="Use stdin redirection to tty",
                        action="store_true",
                        default=False,
                        required=False)
    parser.add_argument('--replay-no-exit',
                        help="Do not exit on panics/rom returns when replaying logs (for batch analysis)",
                        action="store_true",
                        default=False,
                        required=False)
    if len(sys.argv) == 1:
        sys.argv = sys.argv + ["--help"]

    opts = parser.parse_args()

    #Open files, rebuild if needed
    opts.file[0], dumps = helper.process_files(opts.file[0], opts.rebuild)

    dump_path = os.path.dirname(__file__) + "/romdumps/"

    plusargs = {}
    if opts.plusargs:
        for group in opts.plusargs:
            for a in group:
                ret = parse("+{}={}", a)
                if ret:
                    plusargs[ret[0]] = ret[1]
                    continue
                ret = parse("+{}", a)
                if ret:
                    plusargs[ret[0]] = True

    chip, term, reset = helper.create_core_stuff_from_options(opts)
    if not term:
        return 1
        
    term.plusargs = plusargs
    try:
        romdump = open(dump_path + chip.romdump, "r")
        term.add_dumps({'rom' : romdump})
    except:
        pass

    if opts.log:
        term.logstream = opts.log

    try:
        reset.resetToHost()
    except:
        print("WARN: Reset method doesn't support HOST mode switching")
        print("WARN: If things don't work - check jumpers!")
        reset.reset()

    term.add_binaries(opts.file)
    term.add_dumps(dumps)
    term.replay_till_the_end = opts.replay_no_exit

    return term.loop(opts.stdin)



from rumboot.chipDb import ChipDb
from rumboot.ImageFormatDb import ImageFormatDb
from rumboot.resetSeq import ResetSeqFactory
from rumboot.cmdline import arghelper
from rumboot.terminal import terminal
from rumboot.server import server
import os
import argparse
import rumboot_xrun
import rumboot
from parse import *

def cli():
    resets  = ResetSeqFactory("rumboot.resetseq")
    formats = ImageFormatDb("rumboot.images")
    chips   = ChipDb("rumboot.chips")

    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description="rumboot-daemon {} - Collaborative board access daemon\n".format(rumboot.__version__) +
                                    rumboot.__copyright__)
    helper = arghelper()
    helper.add_terminal_opts(parser)
    helper.add_file_handling_opts(parser)
    helper.add_resetseq_options(parser, resets)
    parser.add_argument("-L", "--listen",
                        help="Specify address:port to listen (default 0.0.0.0:10000)",
                        nargs=1, metavar=('listen'), default=["0.0.0.0:10000"],
                        required=False)
    parser.add_argument("-m", "--max-usage-time",
                        help="Specify maximum time a client can use the board before being kicked (seconds)",
                        default = 360,
                        type = int,
                        required=False)
    parser.add_argument("--execute-binaries-from-temp-dir",
                        help="Super-special debug mode. Mock a bootrom, receive upload binaries and exec them",
                        action="store_true",
                        required=False)

    opts = parser.parse_args()
    if opts.execute_binaries_from_temp_dir:
        opts.port = "loop://"

    chip, term, reset = helper.create_core_stuff_from_options(opts)

    c = helper.detect_chip_type(opts, chips, formats)

    if opts.log:
        term.logstream = opts.log

    srv = server(term, opts.listen[0], opts.max_usage_time, opts.execute_binaries_from_temp_dir)
    srv.set_reset_seq(reset)

    if "file" in opts:
        srv.preload_binaries(opts.file)

    return srv.loop()
    



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'testing_gui_testing_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.3
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_TestingDialog(object):
    def setupUi(self, TestingDialog):
        TestingDialog.setObjectName("TestingDialog")
        TestingDialog.resize(838, 593)
        self.verticalLayout = QtWidgets.QVBoxLayout(TestingDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.test_progress_bar = QtWidgets.QProgressBar(TestingDialog)
        self.test_progress_bar.setObjectName("test_progress_bar")
        self.verticalLayout.addWidget(self.test_progress_bar)
        self.test_name_line_edit = QtWidgets.QLineEdit(TestingDialog)
        self.test_name_line_edit.setFocusPolicy(QtCore.Qt.NoFocus)
        self.test_name_line_edit.setAlignment(QtCore.Qt.AlignCenter)
        self.test_name_line_edit.setReadOnly(True)
        self.test_name_line_edit.setObjectName("test_name_line_edit")
        self.verticalLayout.addWidget(self.test_name_line_edit)
        self.log_plain_text_edit = QtWidgets.QPlainTextEdit(TestingDialog)
        self.log_plain_text_edit.setUndoRedoEnabled(False)
        self.log_plain_text_edit.setReadOnly(True)
        self.log_plain_text_edit.setObjectName("log_plain_text_edit")
        self.verticalLayout.addWidget(self.log_plain_text_edit)
        self.dialog_button_box = QtWidgets.QDialogButtonBox(TestingDialog)
        self.dialog_button_box.setOrientation(QtCore.Qt.Horizontal)
        self.dialog_button_box.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel)
        self.dialog_button_box.setCenterButtons(True)
        self.dialog_button_box.setObjectName("dialog_button_box")
        self.verticalLayout.addWidget(self.dialog_button_box)

        self.retranslateUi(TestingDialog)
        self.dialog_button_box.accepted.connect(TestingDialog.accept)
        self.dialog_button_box.rejected.connect(TestingDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(TestingDialog)

    def retranslateUi(self, TestingDialog):
        _translate = QtCore.QCoreApplication.translate
        TestingDialog.setWindowTitle(_translate("TestingDialog", "Процесс тестирования"))
        self.test_progress_bar.setFormat(_translate("TestingDialog", "%v/%m"))



import io
import re
import copy

class PartitionBase():
    name = "invalid"
    erase_size = 0
    read_size = 0
    write_size = 0
    size = 0
    offset = 0
    partitions = {}
    last_added_partition_end = 0

    def __init__(self, terminal):
        self.terminal = terminal

    def __getitem__(self, key):
        return self.partitions[key]

    def save_partitions(self):
        raise Exception(f"FATAL: Saving partition table is not implemented for this device")

    def load_partitions(self):
        raise Exception(f"FATAL: Loading partition table is not implemented for this device")

    def erase(self, offset=0, length=-1, callback=None):
        off = 0
        if length < 0:
            length = self.size

        ln = length

        if length % self.erase_size > 0:
            raise Exception(f"FATAL: Erase length is not aligned by erase_size boundary")
        if offset % self.erase_size > 0:
            raise Exception(f"FATAL: Erase offset is not aligned by erase_size boundary")

        self._erase(self.offset + offset, length, callback=callback)

    def write(self, fd, offset, length = -1, callback = None):
        if length < 0:
            ln = self.stream_size(fd)
        else:
            ln = length

        pad = 0
        if ln % self.write_size:
            pad += self.write_size - (ln % self.write_size)

        #TODO: Do we really have to handle padding here?
        if pad > 0:
            #print("WARNING: padding data, it eats up ram")
            fd = fd.read()
            while pad > 0:
                fd +=b"\x00"
                pad -= 1
            fd = io.BytesIO(fd)
            ln = self.stream_size(fd)

        if (offset + self.offset) % self.write_size:
            raise Exception("Misaligned write offset")
        
        initial_pos = fd.tell()

        if not ln > 0:
            raise Exception(f"FATAL: Length can't be {ln}")
        
        return self._write(fd, self.offset + offset, ln, callback)

    def read(self, fd, offset, length = -1, callback = None):
        if offset + self.offset % self.read_size:
            raise Exception("Misaligned read")

        if length % self.read_size:
            raise Exception("Misaligned read size")

        if length < 0:
            ln = self.size - self.offset - offset
        else:
            ln = length

        return self._read(fd, self.offset + offset, ln, callback)

    # offset: positive - byte offset from start of device
    #         negative - byte offset from the end of device
    #         None     - make it follow last added partition

    def add_partition(self, name, offset = 0, length=-1):

        partitions = self.partitions
        partitions[name]        = copy.copy(self)
        partitions[name].name   = name

        if offset is None:
            offset = self.last_added_partition_end
        elif offset < 0:
            offset = self.size - offset

        if offset % self.erase_size:
            raise Exception(f"FATAL: Partition {name} offset is not aligned by erase_size boundary")

        partitions[name].offset = offset

        maxsize = self.size - offset

        if length == -1 or length > maxsize:
            partitions[name].size = maxsize
        else:
            partitions[name].size = length

        if partitions[name].size % self.erase_size:
            raise Exception(f"FATAL: Partition {name} size is not aligned by erase_size boundary")
        
        self.last_added_partition_end = offset + partitions[name].size

        return partitions[name]

    def stream_size(self, stream):
        pos = stream.tell()
        stream.seek(0,2)
        ln = stream.tell()
        stream.seek(pos)
        return ln - pos

    def fromfile(self, path):
        if type(path) == str:
            path = open(path, "rb")
        return self.write(0, path)

    def tofile(self, path):
        if type(path) == str:
            path = open(path, "wb+")
        self.read(0, self.size, path)
        path.close()

    @staticmethod
    def sizeof_fmt(num, suffix='B'):
        for unit in ['','Ki','Mi','Gi','Ti','Pi','Ei','Zi']:
            if abs(num) < 1024.0:
                return "%3.1f%s%s" % (num, unit, suffix)
            num /= 1024.0
        return "%.1f%s%s" % (num, 'Yi', suffix)

    def dump(self):
        print(f"Device: {self.name} part: {self.part} size: {self.sizeof_fmt(self.size)} erase_size: {self.sizeof_fmt(self.erase_size)} write_size: {self.sizeof_fmt(self.write_size)}")

    def dump_parts(self):
        for k,v in self.partitions.items():
            v.dump()
            
#Basic flash device with access functions
class FlashDeviceBase():
    skip_erase_before_write = False

    def _read(self, fd, offset, length, callback = None):
        raise Exception("NOT IMPLEMENTED")

    def _write(self, fd, offset, length, callback = None):
        raise Exception("NOT IMPLEMENTED")

    def _erase(self, offset=0, length=-1, callback = None):
        raise Exception("NOT IMPLEMENTED")

    def switchbaud(self, newbaud):
        raise Exception("Not implemented")

    def saveenv(self):
        raise Exception("Not implemented")

    def env(self, key, value=None):
        raise Exception("Not implemented")



import yaml
from os import listdir
from os.path import join, isfile, dirname


class ChipDb:

    chips_by_name = {}
    chips_by_id = {}
    params_cfg_name = "chips_parameters.yml"

    def __init__(self, path):
        if len(self.chips_by_name) == 0 or len(self.chips_by_id) == 0:
            path = path.split(".")
            root = dirname(dirname(__file__))
            path = join(root, *path)
            with open(join(path, self.params_cfg_name), 'r') as f:
                params = yaml.safe_load(f)

            yamles = [
                f for f in listdir(path) if isfile(join(path, f)) and \
                f.endswith(".yml") and f != self.params_cfg_name
            ]

            for yml in yamles:
                with open(join(path, yml), 'r') as f:
                    chips = yaml.load_all(f, Loader = yaml.SafeLoader)
                    for chip in chips:
                        excess_attrs = \
                            set(chip.__dict__) - set(params['required'] + params['optional'])
                        if excess_attrs:
                            raise ExcessAttrsError(excess_attrs, chip.chip_id, yml)

                        missing_attrs = \
                            set(params['required']).difference(set(chip.__dict__))
                        if missing_attrs:
                            raise MissingAttrsError(missing_attrs, chip.chip_id, yml)

                        excess_hacks = set(chip.hacks) - set(params['hacks'])
                        if excess_hacks:
                            raise ExcessHacksError(excess_hacks, chip.chip_id, yml)

                        missing_hacks = set(params['hacks']).difference(set(chip.hacks))
                        if missing_hacks:
                            raise MissingHacksError(missing_hacks, chip.chip_id, yml)

                        if chip.name in self.chips_by_name:
                            raise MultipleChipConfigError(chip.name, chip.chip_id, yml)
                        self.chips_by_name[chip.name] = chip

                        if chip.chip_id in self.chips_by_id:
                            self.chips_by_id[chip.chip_id].append(chip)
                        else:
                            self.chips_by_id[chip.chip_id] = [chip]

    def __getitem__(self, key):
        try:
            key = int(key)
        except ValueError:
            return self.chips_by_name[key]
        else:
            chips = self.chips_by_id[key]
            if len(chips) > 1:
                raise ChipsWithSameIDError(key)
            return chips[0]


class Chip(yaml.YAMLObject):
    yaml_tag = '!Chip'
    yaml_loader = yaml.SafeLoader


class MultipleChipConfigError(Exception):
    def __init__(self, name, chip_id, filename):
        super().__init__(
            f"Found duplicate config [{filename}] for chip with name [{name}] and ID [{chip_id}]"
        )


class ChipsWithSameIDError(Exception):
    def __init__(self, ID):
        super().__init__(
            f"Found multiple chips for given ID [{ID}]"
        )


class InvalidAttrsError(Exception):
    def __init__(self, chip_id, filename, postfix):
        super().__init__(
            f"Chip with ID [{chip_id}] in file [{filename}] {postfix}"
        )


class MissingHacksError(InvalidAttrsError):
    def __init__(self, hacks, chip_id, filename):
        super().__init__(chip_id, filename, f"has missing hacks [{hacks}]")


class ExcessHacksError(InvalidAttrsError):
    def __init__(self, hacks, chip_id, filename):
        super().__init__(chip_id, filename, f"has excess hacks: {hacks}")


class MissingAttrsError(InvalidAttrsError):
    def __init__(self, attrs, chip_id, filename):
        super().__init__(chip_id, filename, f"has missing attributes: {attrs}")


class ExcessAttrsError(InvalidAttrsError):
    def __init__(self, attrs, chip_id, filename):
        super().__init__(chip_id, filename, f"has excess attributes: {attrs}")



import os
import time
import usb.core
import usb.util
from parse import parse
from rumboot.resetseq.resetSeqBase import base
from requests import post

class hass(base):
    name = "HomeAssistant Switch"
    swap   = False
    supported = ["POWER", "RESET", "HOST"]
    mapping = {
        #defaults to a test switch
        "POWER" : None,
        "RESET" : None,
        "HOST"  : None
    }

    def __init__(self, terminal, opts):
        self.url   = opts["hass_server"]
        self.token = opts["hass_token"]
        self.mapping["POWER"] = opts["hass_power_switch"]
        self.mapping["RESET"] = opts["hass_reset_switch"]
        self.mapping["HOST"]  = opts["hass_host_switch"]

        super().__init__(terminal, opts)

    def __setitem__(self, key, value):
        super().__setitem__(key, value)
        if self.mapping[key] is None:
            return 

        if value:
            url = self.url + "/api/services/switch/turn_on"
        else:
            url = self.url + "/api/services/switch/turn_off"

        headers = {
            "Authorization": "Bearer " + self.token,    
            "content-type": "application/json",
        }

        data = '{"entity_id": "%s"}' % self.mapping[key]
        response = post(url, headers=headers, data = data)

    def get_options(self):
        return {
                "hass-power-switch" : {
                    "help" : "Home Assistant power switch",
                    "default" : "switch.0x00158d0002c7faa0",
                    "action"  : 'store_true'
                },
                "hass-reset-switch" : {
                    "help" : "Home Assistant reset switch",
                    "default" : False,
                },
                "hass-host-switch" : {
                    "help" : "Home Assistant host switch",
                    "default" : False,
                },
                "hass-token" : {
                    "help" : "Secret token",
                    # Defaults to a test token
                    "default" : "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJjZDUyMGYzZGUxZWQ0YTMzOWI4ZDY0ODQ3OWI0ZjE4ZiIsImlhdCI6MTYyOTU0NDg3NiwiZXhwIjoxOTQ0OTA0ODc2fQ.bJIj5VU_tNQLG6TXk1rkbA-BFpe0mYEsGfoWS4olzXs"
                },
                "hass-server" : {
                    "help" : "Hass server URL",
                    # Defaults to a test server
                    "default" : "http://homeblade:8123"
                }
            }


import sys
import os
from PyQt5 import QtWidgets
from PyQt5.QtCore import QPoint, QSize, QSettings, Qt, QTimer, QObject, QThread, pyqtSlot, pyqtSignal
from PyQt5.QtGui import QTextCursor
from rumboot.testing.test_desc import *
from rumboot.testing.core import *
from rumboot.testing.executor import TestExecutor
from rumboot.testing.user_interaction import UserInteraction
from rumboot.testing.testing_gui_main_window import Ui_MainWindow
from rumboot.testing.testing_gui_testing_dialog import Ui_TestingDialog


USER_INTERACTION_TYPE_MESSAGE = 0
USER_INTERACTION_TYPE_YES_NO = 1
USER_INTERACTION_TYPE_OPTION = 2

class UserInteractionRequest:

    def __init__(self, test, type, text, options=None):
        self.test = test
        self.type = type
        self.text = text
        self.options = options
        self.result = None


class TestExecutorWrapper(QObject, UserInteraction):

    finished = pyqtSignal()
    progress = pyqtSignal(int)
    log_str = pyqtSignal(str)
    user_interaction = pyqtSignal(object)

    def __init__(self, test_desc_list, test_context, board_number):
        super().__init__()
        self._test_desc_list = test_desc_list
        self._board_number = board_number
        self._executor = TestExecutor(test_context, self, log_func=self._log_func)
        self._stop_request = False
        self._thread = QThread()
        self.moveToThread(self._thread)
        self._thread.started.connect(self._thread_proc)

        if self._board_number:
            self._test_and_mkdir("logs")
            self._board_log_path = os.path.join("logs", f"board_{self._board_number}")
            self._test_and_mkdir(self._board_log_path)


    # UserInteraction
    def request_message(self, test, text):
        request = UserInteractionRequest(test, USER_INTERACTION_TYPE_MESSAGE, text)
        self.user_interaction.emit(request)

    # UserInteraction
    def request_yes_no(self, test, text):
        request = UserInteractionRequest(test, USER_INTERACTION_TYPE_YES_NO, text)
        request.result = False
        self.user_interaction.emit(request)
        return request.result

    # UserInteraction
    def request_option(self, test, text, options):
        request = UserInteractionRequest(test, USER_INTERACTION_TYPE_OPTION, text, options)
        request.result = 0
        self.user_interaction.emit(request)
        return request.result

    def start_testing_nonblocking(self):
        self._stop_request = False
        self._thread.start()

    def terminate_testing_nonblocking(self):
        prev_stop_request = self._stop_request
        self._stop_request = True
        return prev_stop_request

    @pyqtSlot()
    def _thread_proc(self):
        test_index = 0
        for test_desc in self._test_desc_list:
            if self._stop_request:
                break
            self.progress.emit(test_index)
            test_index += 1
            self._executor.exec_test(test_desc)
            self._save_log(test_desc)
        self._thread.quit() # quit from default message loop
        self.finished.emit()

    def _log_func(self, text):
        self.log_str.emit(text)

    def _save_log(self, test_desc):
        if self._board_number == 0:
            return
        with open(os.path.join(self._board_log_path, f"{test_desc.full_name}.log"), "w") as file:
            file.write(test_desc.log_text)

    def _test_and_mkdir(self, path):
        if not os.path.exists(path):
            os.mkdir(path)
        elif not os.path.isdir(path):
            raise Exception(f"{path} is not a directory")


class UserOptionDialog(QtWidgets.QDialog):

    def __init__(self, parent, request):
        super().__init__(parent)
        self._request = request
        self.setWindowTitle("Запрос к пользователю")

        self.layout = QtWidgets.QVBoxLayout()
        self.layout.addWidget(QtWidgets.QLabel(f"Тест: {self._request.test.full_name}\n\n{self._request.text}:\n"))
        index = 0
        for option in self._request.options:
            button = QtWidgets.QPushButton(option)
            self.layout.addWidget(button)
            button.clicked.connect(self._make_func_accept_option(index))
            index += 1
        self.setLayout(self.layout)

    def _make_func_accept_option(self, index):
        def func_template():
            self._request.result = index
            self.accept()
        return func_template


class TestingDialog(QtWidgets.QDialog, Ui_TestingDialog):

    def __init__(self, parent, test_desc_list, test_context, board_number):
        super().__init__(parent)
        self._test_desc_list = test_desc_list
        self._executor_wrapper = TestExecutorWrapper(test_desc_list, test_context, board_number)
        self._executor_wrapper.finished.connect(self.accept, type=Qt.BlockingQueuedConnection)
        self._executor_wrapper.progress.connect(self._executor_wrapper_progress, type=Qt.BlockingQueuedConnection)
        self._executor_wrapper.log_str.connect(self._executor_wrapper_log_str, type=Qt.BlockingQueuedConnection)
        self._executor_wrapper.user_interaction.connect(self._executor_wrapper_user_interaction, type=Qt.BlockingQueuedConnection)

        self.setupUi(self)
        self.test_progress_bar.setRange(0, len(self._test_desc_list))

        QTimer.singleShot(0, self._start_testing)

    def done(self, code):
        if code == QtWidgets.QDialog.Accepted:
            super().done(code)
        else:
            if not self._executor_wrapper.terminate_testing_nonblocking():
                self._executor_wrapper_log_str("\n*** Stop request has been send. Wait for the end of the current test...\n")
                self.dialog_button_box.setEnabled(False)

    @pyqtSlot()
    def _start_testing(self):
        self._executor_wrapper.start_testing_nonblocking()

    @pyqtSlot(int)
    def _executor_wrapper_progress(self, test_index):
        self.test_progress_bar.setValue(test_index + 1)
        self.test_name_line_edit.setText(self._test_desc_list[test_index].full_name)

    @pyqtSlot(str)
    def _executor_wrapper_log_str(self, text):
        self.log_plain_text_edit.moveCursor(QTextCursor.End)
        self.log_plain_text_edit.insertPlainText(text)

    @pyqtSlot(object)
    def _executor_wrapper_user_interaction(self, request):
        if request.type == USER_INTERACTION_TYPE_MESSAGE:
            QtWidgets.QMessageBox.information(self, "Запрос к пользователю", f"Тест: {request.test.full_name}\n\n{request.text}")
        elif request.type == USER_INTERACTION_TYPE_YES_NO:
            answer = QtWidgets.QMessageBox.question(self, "Запрос к пользователю", f"Тест: {request.test.full_name}\n\n{request.text}", QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
            request.result = answer == QtWidgets.QMessageBox.Yes
        elif  request.type == USER_INTERACTION_TYPE_OPTION:
            dialog = UserOptionDialog(self, request)
            dialog.exec_()
        else:
            raise Exception("Unknown user interaction request type")


class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):

    def __init__(self, test_registry, test_context):
        super().__init__()
        self._test_registry = test_registry
        self._test_context = test_context
        self._test_items = []

        self.setupUi(self)
        self._setup_test_tree()
        self.test_selected_button.clicked.connect(self._test_selected_button_clicked)
        self.test_failed_button.clicked.connect(self._test_failed_button_clicked)
        self.exit_action.triggered.connect(self.close)
        self.reset_status_log_action.triggered.connect(self._reset_status_log_action_triggered)
        self.about_qt_action.triggered.connect(lambda : QtWidgets.QMessageBox.aboutQt(self))

        self._settings = QSettings("gui.ini", QSettings.IniFormat)
        self._settings_resore()

        self._update_chip()
        self._reload_test_tree()
        self._update_current_test_info()
        self.current_test_tab_widget.setCurrentIndex(0)

    def closeEvent(self, event):
        self._settings_save()

    def _setup_test_tree(self):
        self.test_tree.setColumnCount(2)
        self.test_tree.setHeaderLabels(["Тест", "Статус"])
        self.test_tree.header().setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
        self.test_tree.itemChanged.connect(self._test_tree_update_item)
        self.test_tree.itemSelectionChanged.connect(self._update_current_test_info)

    def _settings_resore(self):
        self._settings.beginGroup("Main")
        self.board_number_auto_inc_action.setChecked(self._settings.value("board_number_auto_inc", 0, int) != 0)
        self.board_number_spin_box.setValue(self._settings.value("board_number", 0, int))
        self._settings.endGroup()

        self._settings.beginGroup("MainWindow")
        self.resize(self._settings.value("size", self.size(), QSize))
        self.move(self._settings.value("pos", self.pos(), QPoint))
        self._settings.endGroup()

    def _settings_save(self):
        self._settings.beginGroup("Main")
        self._settings.setValue("board_number_auto_inc", 1 if self.board_number_auto_inc_action.isChecked() else 0)
        self._settings.setValue("board_number", self.board_number_spin_box.value())
        self._settings.endGroup()

        self._settings.beginGroup("MainWindow")
        self._settings.setValue("size", self.size())
        self._settings.setValue("pos", self.pos())
        self._settings.endGroup()

    def _update_chip(self):
        self.chip_name_line_edit.setText(f"{self._test_context.chip.name} ({self._test_context.chip.part})")

    def _reload_test_tree(self):
        update_suitable(self._test_registry, self._test_context)
        self.test_tree.clear()
        self._test_items = []
        self._reload_test_tree_recursive(self._test_registry.all_tests, None)
        self._update_tests_status()
        self.test_tree.expandAll()

    def _reload_test_tree_recursive(self, tests, parent):
        for key, value in tests.items():
            item = QtWidgets.QTreeWidgetItem(self.test_tree if parent == None else parent)
            if isinstance(value, TestDesc):
                item.test_desc = value
                self._test_items.append(item)
                item.setText(0, value.name)
                item.setCheckState(0, Qt.Unchecked)
                if value.suitable:
                    item.setFlags(item.flags() | Qt.ItemIsUserCheckable)
                else:
                    item.setFlags(item.flags() & ~Qt.ItemIsUserCheckable)
                    item.setForeground(0, Qt.gray)
            else:
                item.test_desc = None
                item.setText(0, key)
                item.setCheckState(0, Qt.Unchecked)
                item.setFlags(item.flags() | Qt.ItemIsUserCheckable)
                self._reload_test_tree_recursive(value, item)

    def _update_current_test_info(self):
        self.current_test_log_plain_text_edit.setPlainText(None)
        self.current_test_log_plain_text_edit.hide()
        self.current_test_description_label.setText(None)
        self.current_test_description_label.hide()

        test_desc = None
        if self.test_tree.selectedItems():
            item = self.test_tree.selectedItems()[0]
            test_desc = item.test_desc

        if test_desc:
            if test_desc.log_text:
                self.current_test_log_plain_text_edit.setPlainText(test_desc.log_text)
                self.current_test_log_plain_text_edit.show()
            if "description" in test_desc.params:
                self.current_test_description_label.setText(test_desc.params["description"])
            else:
                self.current_test_description_label.setText("<Нет описания теста>")
            self.current_test_description_label.show()

    @pyqtSlot()
    def _update_tests_status(self):
        for item in self._test_items:
            item.setForeground(1, Qt.black if item.test_desc.suitable else Qt.gray)
            if item.test_desc.status == TEST_STATUS_NOT_EXECUTED:
                item.setText(1, "Не запускался")
                item.setBackground(1, Qt.gray if item.test_desc.suitable else Qt.white)
            elif item.test_desc.status == TEST_STATUS_PASSED:
                item.setText(1, "Пройден")
                item.setBackground(1, Qt.green)
            elif item.test_desc.status == TEST_STATUS_FAULT:
                item.setText(1, "Ошибка")
                item.setBackground(1, Qt.red)
            else:
                raise Exception("Unknown test status")

    @pyqtSlot(QtWidgets.QTreeWidgetItem, int)
    def _test_tree_update_item(self, item, column):
        if not item.test_desc:
            for i in range(item.childCount()):
                child = item.child(i)
                if not child.test_desc or child.test_desc.suitable:
                    child.setCheckState(0, item.checkState(0))

    @pyqtSlot()
    def _test_selected_button_clicked(self):
        test_desc_list = []
        for item in self._test_items:
            if item.checkState(0) == Qt.Checked:
                test_desc = item.test_desc
                test_desc.status = TEST_STATUS_NOT_EXECUTED
                test_desc.log_text = None
                test_desc_list.append(test_desc)
        self._test_list(test_desc_list)

    @pyqtSlot()
    def _test_failed_button_clicked(self):
        test_desc_list = []
        for item in self._test_items:
            test_desc = item.test_desc
            if test_desc.status == TEST_STATUS_FAULT:
                test_desc_list.append(test_desc)
        self._test_list(test_desc_list)

    def _test_list(self, test_desc_list):
        if not test_desc_list:
            QtWidgets.QMessageBox.critical(self, "Ошибка", "Нет выбранных тестов")
            return
        self._update_tests_status()
        self._update_current_test_info()
        testing_dialog = TestingDialog(self, test_desc_list, self._test_context, self.board_number_spin_box.value())
        testing_dialog.exec_()
        self._update_tests_status()
        self._update_current_test_info()
        if self.board_number_auto_inc_action.isChecked():
            if all([x.test_desc.status == TEST_STATUS_PASSED for x in self._test_items if x.test_desc.suitable]):
                self.board_number_spin_box.setValue(self.board_number_spin_box.value() + 1)
                self._settings_save()
                QtWidgets.QMessageBox.information(self, "Информация", "Все тесты пройдены, номер платы увеличен на 1")

    @pyqtSlot()
    def _reset_status_log_action_triggered(self):
        for item in self._test_items:
            test_desc = item.test_desc
            test_desc.status = TEST_STATUS_NOT_EXECUTED
            test_desc.log_text = None
        self._update_tests_status()
        self._update_current_test_info()


def start_testing_gui(test_registry, test_context):
    app = QtWidgets.QApplication(sys.argv)
    window = MainWindow(test_registry, test_context)
    window.show()
    app.exec_()
